@@ -10,6 +10,16 @@ BEGIN;
 
 --\set ON_ERROR_STOP
 
+COPY cc_collector_platform_mst (platform_id, platform_name, order_no) FROM stdin (DELIMITER ',');
+AWS,Amazon Web Services,110
+\.
+
+COMMIT;
+
+BEGIN;
+
+--\set ON_ERROR_STOP
+
 COPY cc_collector_sub_platform_mst (sub_platform_id, sub_platform_name, type, order_no) FROM stdin (DELIMITER ',');
 AWS,Amazon Web Services,Cloud,110
 \.
