@@ -22,7 +22,7 @@ BEGIN;
 
 --\set ON_ERROR_STOP
 --\http://docs.aws.amazon.com/general/latest/gr/rande.html
-DELETE FROM cc_cloud_endpoint WHERE ep_type = 'ec2';
+DELETE FROM cc_cloud_endpoint WHERE cloud_service_id = 'AWS';
 
 COMMIT;
 
@@ -54,6 +54,14 @@ BEGIN;
 
 --\set ON_ERROR_STOP
 
+DELETE FROM cc_collector_platform_mst WHERE platform_id = 'AWS';
+
+COMMIT;
+
+BEGIN;
+
+--\set ON_ERROR_STOP
+
 DELETE FROM cc_collector_sub_platform_mst WHERE sub_platform_id = 'AWS';
 
 COMMIT;
\ No newline at end of file
