@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2013 NTT DATA Corporation
+Copyright (C) 2014 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License
@@ -18,7 +18,6 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.persistence.EntityExistsException;
 import javax.persistence.Query;
@@ -55,7 +54,6 @@ import com.clustercontrol.cloud.bean.StopInstanceRequest;
 import com.clustercontrol.cloud.bean.Tag;
 import com.clustercontrol.cloud.commons.CloudConstants;
 import com.clustercontrol.cloud.commons.CloudPropertyConstants;
-import com.clustercontrol.cloud.commons.Objects;
 import com.clustercontrol.cloud.commons.PropertyContract;
 import com.clustercontrol.cloud.dao.CloudInstanceBackupDao;
 import com.clustercontrol.cloud.dao.CloudInstanceDao;
@@ -81,38 +79,10 @@ import com.clustercontrol.repository.session.RepositoryControllerBean;
 @Transactional
 public class InstanceOperator extends ResourceOperatorBase implements IInstanceOperator {
 	
-	private static class CriticalPoint {
-		private AtomicInteger syncCount = new AtomicInteger();
-
-		public void enter() {
-			syncCount.incrementAndGet();
-		}
-
-		public void leave() {
-			syncCount.decrementAndGet();
-		}
-
-		public boolean isEntering() {
-			return syncCount.get() != 0;
-		}
-	}
-
 	private boolean update = CloudPropertyConstants.autoupdate_instance.match(PropertyContract.on);
 	private boolean regist = CloudPropertyConstants.autoregist_instance.match(PropertyContract.on);
 	private boolean relation = CloudPropertyConstants.autoregist_scope_relation.match(PropertyContract.on);
 
-	private static Map<Object, CriticalPoint> criticalPointMap = new HashMap<>();
-
-	private static synchronized CriticalPoint getCriticalPoint(Object key) {
-		CriticalPoint cp = criticalPointMap.get(key);
-		if (cp == null) {
-			cp = new CriticalPoint();
-			criticalPointMap.put(key, cp);
-		}
-		return cp;
-	}
-
-
 	public InstanceOperator() throws CloudManagerFault {
 	}
 
@@ -217,7 +187,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 	}
 
 	protected CloudInstance internalCreateInstance(NodeDetail nodeDetail, String flavor, String imageId, String zone, String instanceDetail, String templateId, List<Tag> tags) throws CloudManagerFault, InvalidRole {
-		Instance instance = getResourceManagement().createInstance(nodeDetail.getFacilityName(), flavor, imageId, zone, instanceDetail, tags);
+		Instance instance = getResourceManagement().createInstance(nodeDetail.getFacilityId(), nodeDetail.getFacilityName(), flavor, imageId, zone, instanceDetail, tags);
 		CloudInstanceDao dao = createCloudInstanceDao(instance, templateId);
 		CloudInstance cloudInstance = createCloudInstance(nodeDetail, dao, instance);
 
@@ -262,12 +232,13 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 		}
 
 		Instance instance = getResourceManagement().restoreInstance(
+				request.getNodeDetail().getFacilityId(),
 				request.getInstanceBackupId(), nodeDetail.getFacilityName(),
 				request.getFlavor(), request.getZone(),
 				request.getInstanceDetail(), request.getTags());
 		CloudInstanceDao dao = createCloudInstanceDao(instance, null);
 		CloudInstance cloudInstance = createCloudInstance(nodeDetail, dao,instance);
-
+		
 		InstanceMonitorService.getSingleton().startMonitor(
 				instance.getInstanceId(), cloudInstance.getFacilityId(),
 				SessionService.current().getContext(),
@@ -335,6 +306,11 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 					throw ErrorCode.HINEMOS_MANAGER_ERROR.cloudManagerFault(e);
 				}
 			}
+			// CloudInstance に対するノードは、スキーマ上 ユニークなので、 facilityId をキーにバックアップ情報を取得しても問題ない。
+			List<CloudInstanceBackupDao> backups = em.findByFilter(CloudInstanceBackupDao.class, new Filter("backupedData.facilityId", nodeDetail.getFacilityId()));
+			for (CloudInstanceBackupDao backup: backups) {
+				backup.getBackupedData().setInstanceId(dao.getInstanceId());
+			}
 		}
 
 		if (nodeInfo == null) {
@@ -595,7 +571,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 			}
 		}
 
-		UpdateResult result = internalUpdate(cloudInstanceDao, instance, getCriticalPoint(getCloudRegion().getRegion()).isEntering());
+		UpdateResult result = internalUpdate(cloudInstanceDao, instance, CriticalPoint.getCriticalPoint(getAccountResource().getAccountId(), getCloudRegion().getRegion()).isEntering());
 
 		return result;
 	}
@@ -614,7 +590,8 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 			scope.complete();
 		}
 		catch (CloudManagerFault e) {
-			CloudMessageUtil.notify_AutoUpadate_Error_InstanceOperator(cloudInstanceDao.getAccountResourceId(), cloudInstanceDao.getInstanceId(), e);
+			String instanceId = cloudInstanceDao != null ? cloudInstanceDao.getInstanceId(): instance.getInstanceId();
+			CloudMessageUtil.notify_AutoUpadate_Error_InstanceOperator(getAccountResource().getAccountResourceId(), instanceId, e);
 		}
 		
 		return result;
@@ -726,7 +703,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 		}
 		catch (CloudManagerFault e) {
 			if (ErrorCode.CLOUDINSTANCE_DUPLICATE_NODE.match(e) || ErrorCode.HINEMOS_MANAGER_ERROR.match(e)) {
-				CloudMessageUtil.notify_AutoUpadate_Error_InstanceOperator(cloudInstanceDao.getAccountResourceId(), cloudInstanceDao.getInstanceId(), e);
+				CloudMessageUtil.notify_AutoUpadate_Error_InstanceOperator(getAccountResource().getAccountResourceId(), cloudInstanceDao.getInstanceId(), e);
 			}
 			else {
 				throw e;
@@ -774,9 +751,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 
 	private CloudInstanceDao createCloudInstanceDao(Instance instance, String templateId) throws CloudManagerFault {
 		try {
-			getCriticalPoint(
-					new Objects(getCloudUser().getAccountResourceId(),
-							getCloudRegion().getRegion())).enter();
+			CriticalPoint.getCriticalPoint(getAccountResource().getAccountResourceId(), getCloudRegion().getRegion()).enter();
 
 			// DB に追加する情報を作成。
 			CloudInstanceDao dao = new CloudInstanceDao();
@@ -796,10 +771,10 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 				return dao;
 			}
 			catch (EntityExistsException e) {
-				throw ErrorCode.CLOUDSTORAGE_ALREADY_EXIST.cloudManagerFault(dao.getInstanceId());
+				throw ErrorCode.CLOUDINSTANCE_ALREADY_EXIST.cloudManagerFault(dao.getInstanceId());
 			}
 		} finally {
-			getCriticalPoint(new Objects(getCloudUser().getAccountResourceId(), getCloudRegion().getRegion())).leave();
+			CriticalPoint.getCriticalPoint(getAccountResource().getAccountResourceId(), getCloudRegion().getRegion()).leave();
 		}
 	}
 
@@ -879,9 +854,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 		List<UpdateAllResult.InstanceMapping> onlyAws = new ArrayList<>();
 		List<CloudInstanceDao> onlyCloud = new ArrayList<CloudInstanceDao>();
 
-		boolean entering = getCriticalPoint(
-				new Objects(getCloudUser().getAccountResourceId(),
-						getCloudRegion().getRegion())).isEntering();
+		boolean entering = CriticalPoint.getCriticalPoint(getAccountResource().getAccountResourceId(), getCloudRegion().getRegion()).isEntering();
 
 		// AWS にも Hinemos DB にも登録されている。
 		for (UpdateAllResult.InstanceMapping instanceMapping : instanceMappings) {
@@ -896,7 +869,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 			UpdateResult updateResult = transactionalInternalUpdate(null, i, entering);
 
 			// ストレージの自動アタッチ。
-			if (updateResult.cloudInstance != null) {
+			if (updateResult != null && updateResult.cloudInstance != null) {
 				onlyAws.add(new UpdateAllResult.InstanceMapping(updateResult.awsInstance, updateResult.cloudInstance));
 				getLogger().debug("AWS : " + "instanceId=" + i.getInstanceId());
 			}
@@ -905,7 +878,7 @@ public class InstanceOperator extends ResourceOperatorBase implements IInstanceO
 		for (CloudInstanceDao cs : CloudInstanceDaos) {
 			UpdateResult updateResult = transactionalInternalUpdate(cs, null, entering);
 
-			if (updateResult.cloudInstance != null) {
+			if (updateResult != null && updateResult.cloudInstance != null) {
 				onlyCloud.add(updateResult.cloudInstance);
 			}
 		}
